@@ -1,12 +1,13 @@
-__version__ = "0.1.0"
+__version__ = "0.2.0"
 
 from .symbol import Symbol, register_type
 from .expression import Expr, Head, symbol
 from .macro import Macro
+from .ast import parse
 from functools import wraps
 
 __all__ = ["Symbol", "Head", "Expr", "Macro",
-           "symbol", "register_type",
+           "symbol", "register_type", "parse",
            "blocked", "record", "property", "dump", "get_macro"
            ]
 
@@ -24,6 +25,7 @@ def record(obj = None, *, returned_callback = None):
 def property(prop):
     return _MACRO.property(prop)
 
+@wraps(_MACRO.dump)
 def dump() -> str:
     return _MACRO.dump()
 
