@@ -1,5 +1,6 @@
 from __future__ import annotations
 from contextlib import contextmanager
+from copy import deepcopy
 from importlib import import_module
 from functools import partial, wraps
 import inspect
@@ -7,7 +8,7 @@ from collections import UserList
 from typing import Callable, Iterable, Iterator, Any, Union, overload, TypeVar, Hashable
 from types import ModuleType
 
-from .expression import Head, Expr, symbol, EXEC, check_format_mapping
+from .expression import BINOP_MAP, Head, Expr, symbol, EXEC, check_format_mapping
 from .symbol import Symbol
 
 # types
@@ -223,14 +224,54 @@ class Macro(UserList):
         """
         return self.record(property(prop))
     
-    def call_builtin(self, func: Callable, *args, **kwargs):
+    def optimize(self, inplace=False) -> Macro:
         """
-        Call Python builtin function in macro recording mode.
+        Optimize macro readability by deleting unused variables.
+
+        Parameters
+        ----------
+        inplace : bool, default is False
+            If true, macro will be updated by the optimized one.
+
+        Returns
+        -------
+        Macro
+            Optimized macro.
+        """        
+        if not inplace:
+            self = deepcopy(self)
+        expr_map: list[tuple[Symbol, Expr]] = []
+        need = set()
+        for i, expr in enumerate(self):
+            expr: Expr|Symbol
+            if expr.head == Head.assign:
+                # TODO: a, b = func(...) don't work
+                expr_map.append((expr.args[0], i))
+                args = expr.args[1:]
+            else:
+                args = expr.args
+                
+            for arg in args:
+                if isinstance(arg, Expr):
+                    need |= set(a for a in arg.iter_args() if (not a.constant))
+                elif not arg.constant:
+                    need.add(arg)
+                    
+        for sym, i in expr_map:
+            if sym not in need:
+                self[i] = self[i].args[1]
+                
+        return self
+        
+        
+    def call_function(self, func: Callable, *args, **kwargs):
+        """
+        Call function in macro recording mode.
 
         Parameters
         ----------
         func : Callable
-            Builtin function.
+            Function you want to call.
         """        
         with self.blocked():
             out = func(*args, **kwargs)
@@ -333,28 +374,38 @@ class mFunction(mObject):
                 return expr
         elif fname == "__setitem__":
             def make_expr(obj: _O, out, *args):
-                expr = Expr(Head.setitem, [self.to_namespace(obj), args[0], args[1]])
-                if self._last_setval == (Head.setitem, args[0]):
+                target = Expr(Head.getitem, [self.to_namespace(obj), args[0]])
+                expr = Expr(Head.assign, [target, args[1]])
+                if self._last_setval == target:
                     self.macro.pop(-1)
                 else:
-                    self._last_setval = (Head.setitem, args[0])
+                    self._last_setval = target
                 return expr
         elif fname == "__setattr__":
             def make_expr(obj: _O, out, *args):
-                expr = Expr(Head.setattr, [self.to_namespace(obj), Symbol(args[0]), args[1]])
-                if self._last_setval == (Head.setattr, args[0]):
+                target = Expr(Head.getattr, [self.to_namespace(obj), Symbol(args[0])])
+                expr = Expr(Head.assign, [target, args[1]])
+                if self._last_setval == target:
                     self.macro.pop(-1)
                 else:
-                    self._last_setval = (Head.setattr, args[0])
+                    self._last_setval = target
                 return expr
         elif fname == "__delitem__":
             def make_expr(obj: _O, out, *args):
-                expr = Expr(Head.delitem, [self.to_namespace(obj), args[0]])
+                target = Expr(Head.getitem, [self.to_namespace(obj), args[0]])
+                expr = Expr(Head.del_, [target])
                 self._last_setval = None
                 return expr
         elif fname == "__delattr__":
             def make_expr(obj: _O, out, *args):
-                expr = Expr(Head.delattr, [self.to_namespace(obj), Symbol(args[0])])
+                target = Expr(Head.getattr, [self.to_namespace(obj), args[0]])
+                expr = Expr(Head.del_, [target])
+                self._last_setval = None
+                return expr
+        elif fname in BINOP_MAP.keys():
+            op = BINOP_MAP[fname]
+            def make_expr(obj: _O, out, *args):
+                expr = Expr(Head.binop, [op, self.to_namespace(obj), args[0]])
                 self._last_setval = None
                 return expr
         else:
